package com.ejuniper.org;

import javax.xml.namespace.QName;
import javax.xml.soap.*;
import java.io.*;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Random;
import java.util.zip.GZIPInputStream;


public class JuniperExample {
    static String User = "INSERT HERE THE USER PROVIDED BY OUR XML AGENTS";
    static String Pass = "INSERT HERE THE PASS PROVIDED BY OUR XML AGENTS";

    public static void main(String[] args) {
        String soapEndpointUrl = "https://xml-uat.bookingengine.es/WebService/jp/operations/availtransactions.asmx";
        String soapAction = "http://www.juniper.es/webservice/2007/TransferAvail";

        callSoapWebService(soapEndpointUrl, soapAction);
    }

    private static void createSoapEnvelope(SOAPMessage soapMessage) throws SOAPException {
        String StartDATE;
        String EndDATE;

        //Sample Origin and Destination
        String OriginZone = "50277";
        String DestinationZone = "1192";
        String Route = "InOut";

        //Random Dates
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        LocalDateTime localDateTime = LocalDateTime.now();

        //Get random amount of days between 5 and 10
        Random random = new Random();
        int randomAmountOfMonths = random.nextInt(10 - 5 + 1) + 5;
        int randomAmountOfDays = random.nextInt(10 - 5 + 1) + 5;

        LocalDateTime futureLocalDateTime = localDateTime.plusMonths(randomAmountOfMonths);

        StartDATE = futureLocalDateTime.format(dateTimeFormatter);
        EndDATE = futureLocalDateTime.plusDays(randomAmountOfDays).format(dateTimeFormatter);

        SOAPPart soapPart = soapMessage.getSOAPPart();

        String myNamespace = "JP";
        String myNamespaceURI = "http://www.juniper.es/webservice/2007/";

        // SOAP Envelope
        SOAPEnvelope envelope = soapPart.getEnvelope();
        envelope.addNamespaceDeclaration(myNamespace, myNamespaceURI);

        // SOAP Body
        SOAPBody soapBody = envelope.getBody();
        SOAPElement TransferAvail = soapBody.addChildElement("TransferAvail", myNamespace);
        SOAPElement TransferAvailRQ = TransferAvail.addChildElement("TransferAvailRQ", myNamespace);

        TransferAvailRQ.addAttribute(QName.valueOf("Version"),"1.1");
        TransferAvailRQ.addAttribute(QName.valueOf("Language"),"EN");

        // Login Element
        SOAPElement LoginElement = TransferAvailRQ.addChildElement("Login", myNamespace);
        LoginElement.addAttribute(QName.valueOf("Password"), Pass);
        LoginElement.addAttribute(QName.valueOf("Email"), User);

        // Paxes Element
        SOAPElement PaxesElement = TransferAvailRQ.addChildElement("Paxes", myNamespace);
        SOAPElement Pax1Element = PaxesElement.addChildElement("Pax", myNamespace);
        SOAPElement Pax2Element = PaxesElement.addChildElement("Pax", myNamespace);


        Pax1Element.addAttribute(QName.valueOf("IdPax"),"1").addChildElement("Age", myNamespace).addTextNode("30");
        Pax2Element.addAttribute(QName.valueOf("IdPax"),"2").addChildElement("Age", myNamespace).addTextNode("30");

        // HRequest Element
        SOAPElement TransferRequest = TransferAvailRQ.addChildElement("TransferRequest", myNamespace);

        // SearchSegments Element
        SOAPElement SearchSegmentsTransfers = TransferRequest.addChildElement("SearchSegmentsTransfers", myNamespace);

        SOAPElement SearchSegmenTransfers = SearchSegmentsTransfers.addChildElement("SearchSegmenTransfers", myNamespace);
        SearchSegmenTransfers.addAttribute(QName.valueOf("Start"),StartDATE).addAttribute(QName.valueOf("End"),EndDATE);
        SearchSegmenTransfers.addAttribute(QName.valueOf("OriginZone"),OriginZone).addAttribute(QName.valueOf("DestinationZone"),DestinationZone);
        SearchSegmenTransfers.addAttribute(QName.valueOf("Route"),Route);

        // RelPaxesDist Element
        SOAPElement RelPaxesDistElement = TransferRequest.addChildElement("RelPaxesDist", myNamespace).addChildElement("RelPaxDist", myNamespace).addChildElement("RelPaxes", myNamespace);
        RelPaxesDistElement.addChildElement("RelPax", myNamespace).addAttribute(QName.valueOf("IdPax"),"1");
        RelPaxesDistElement.addChildElement("RelPax", myNamespace).addAttribute(QName.valueOf("IdPax"),"2");


        // AdvancedOptions Elements
        SOAPElement AdvancedOptions = TransferAvailRQ.addChildElement("AdvancedOptions", myNamespace);
        AdvancedOptions.addChildElement("ShowBreakdownPrice", myNamespace).addTextNode("true");
        AdvancedOptions.addChildElement("TimeOut", myNamespace).addTextNode("8000");

    }

    private static void callSoapWebService(String soapEndpointUrl, String soapAction) {
        try {
            // Create SOAP Connection
            SOAPConnectionFactory soapConnectionFactory = SOAPConnectionFactory.newInstance();
            SOAPConnection soapConnection = soapConnectionFactory.createConnection();

            // Send SOAP Message to SOAP Server
            SOAPMessage soapResponse = soapConnection.call(createSOAPRequest(soapAction), soapEndpointUrl);

            // GZIP Response Decompression
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            soapResponse.writeTo(out);
            byte[] barr = out.toByteArray();
            InputStream gzipStream = new GZIPInputStream(new ByteArrayInputStream(barr));
            Reader decoder = new InputStreamReader(gzipStream, "UTF-8");
            BufferedReader buffered = new BufferedReader(decoder);
            int n = 0;
            char[] charbuffer = new char[1024];
            Writer w = new StringWriter();
            while ((n = buffered.read(charbuffer)) != -1) {
                w.write(charbuffer,0,n);
            }
            System.out.println(w.toString());

            soapConnection.close();
        } catch (Exception e) {
            System.err.println("\nError occurred while sending SOAP Request to Server!\nMake sure you have the correct endpoint URL and SOAPAction!\n");
            e.printStackTrace();
        }
    }

    private static SOAPMessage createSOAPRequest(String soapAction) throws Exception {
        MessageFactory messageFactory = MessageFactory.newInstance();
        SOAPMessage soapMessage = messageFactory.createMessage();

        createSoapEnvelope(soapMessage);

        MimeHeaders headers = soapMessage.getMimeHeaders();
        headers.addHeader("SOAPAction", soapAction);
        headers.addHeader("Accept-Encoding", "gzip, deflate");

        soapMessage.saveChanges();

        // Print the request message, just for debugging purposes
        System.out.println("Request SOAP Message:");
        soapMessage.writeTo(System.out);

        System.out.println("\n");

        return soapMessage;
    }

}